import regex as re
from typing import List, Dict, Any, Tuple
from .pl import URL_REGEX_PL
from .common import generate_url_regex
from src.detections import UrlDetection
from src.detectors.interface import Detector


class UrlDetector(Detector):
    def __init__(self, language: str = "pl") -> None:
        self._language = language

    def detect(
        self, text: str, annotations: Dict[str, List[Tuple[int, int, Any]]]
    ) -> List[Tuple[int, int, UrlDetection]]:
        return detect_urls(text, self._language)


def detect_urls(text: str, language: str) -> List[Tuple[int, int, UrlDetection]]:
    """
    Detects urls in the text.
    :param text: the text to be searched
    :type text: str
    :param language: the language of the text
    :type language: str
    :return: a list of tuples containing (start, end, entity_type)
    :rtype: List[Tuple[int, int, UrlAnnotation]]
    """
    if language == "pl":
        url_regex = URL_REGEX_PL
    else:
        url_regex = generate_url_regex(language)

    matches = url_regex.finditer(text)
    urls = []
    for match in matches:
        urls.append((match.start(), match.end(), UrlDetection()))

    return urls
