import regex as re
from typing import List, Dict, Any, Tuple
from src.detections import UserDetection
from src.detectors.interface import Detector


class UserDetector(Detector):
    def __init__(self) -> None:
        super().__init__()

    def detect(
        self, text: str, annotations: Dict[str, List[Tuple[int, int, Any]]]
    ) -> List[Tuple[int, int, UserDetection]]:
        return detect_users(text)


USER_REGEX = re.compile(r"\B(?P<username>\@[\w\-]+)")


def detect_users(text: str) -> List[Tuple[int, int, UserDetection]]:
    """
    Detects users in the text.
    :param text: the text to be searched
    :type text: str
    :param language: the language of the text
    :type language: str
    :return: a list of tuples containing (start, end, entity_type)
    :rtype: List[Tuple[int, int, UserAnnotation]]
    """
    matches = USER_REGEX.finditer(text)
    users = []
    for match in matches:
        users.append((match.start(), match.end(), UserDetection()))
    return users
