from typing import Optional
from src.detections import Detection, MorphosyntacticInfoMixin
import random

from src.dictionaries.morphosyntactic.ner_file import NERFileMorphosyntacticDictionary


class NERFileNKJPMorphosyntacticDictionary(NERFileMorphosyntacticDictionary):
    def __init__(
        self, dictionary_path: Optional[str] = None, always_replace=True
    ) -> None:
        super().__init__(dictionary_path, always_replace)

    def get_random_replacement(self, original_entry: Detection) -> Optional[str]:
        original_entry_type = type(original_entry)
        original_entry_type_name = original_entry_type.TYPE_NAME

        result = None

        if issubclass(original_entry_type, MorphosyntacticInfoMixin):
            # THAT IS A HACK FOR NOW FOR CORRUPTED NKJP TAGS IN DICTIONARY
            morpho_tag = ":".join(original_entry.morpho_tag.split(":")[1:])

            if (
                original_entry_type_name in self._dictionary
                and morpho_tag in self._dictionary[original_entry_type_name]
            ):
                result = random.choice(
                    list(
                        self._dictionary[original_entry_type_name][morpho_tag].values()
                    )
                )

        if result is None and self._always_replace:
            random_type = random.choice(list(self._dictionary.keys()))
            random_tag = random.choice(list(self._dictionary[random_type].keys()))

            result = random.choice(
                list(self._dictionary[random_type][random_tag].values())
            )

        return result
