from typing import List, Tuple
from src.detections import (
    Detection,
    NameDetection,
    SurnameDetection,
    StreetNameDetection,
    CityDetection,
    CountryDetection,
    PhoneNumberDetection,
    UrlDetection,
    UserDetection,
    EmailDetection,
    DateDetection,
    TINDetection,
    KRSDetection,
)
from src.string_replacements import replace
from src.replacers.interface import ReplacerInterface


class TagReplacer(ReplacerInterface):
    def __init__(self):
        self.tags_map = {
            NameDetection: "[OSOBA]",
            SurnameDetection: "[OSOBA]",
            StreetNameDetection: "[MIEJSCE]",
            CityDetection: "[MIEJSCE]",
            CountryDetection: "[MIEJSCE]",
            PhoneNumberDetection: "[DIGITS]",
            UrlDetection: "[WWW]",
            UserDetection: "@[USER]",
            EmailDetection: "[MAIL]",
            DateDetection: "[DATE]",
            TINDetection: "[DIGITS]",
            KRSDetection: "[DIGITS]",
        }

    def replace(
        self, text: str, detections: List[Tuple[int, int, Detection]]
    ) -> Tuple[str, List[Tuple[int, int, Detection]]]:

        result = [
            (start, end, self.tags_map.get(type(entity_type), "[OTHER]"))
            for start, end, entity_type in detections
        ]

        return replace(text, result), []
